@@ -5,20 +5,21 @@ import sys
 import os
 import re
 from argparse import ArgumentParser
+from pathlib import Path
 from zipfile import ZipFile
 from datetime import datetime
 from io import BytesIO
-import ssl
 from typing import List, Optional, Tuple, Union
 from urllib.parse import quote
 from urllib.request import urlopen, Request
 from urllib.error import HTTPError, URLError
 from termcolor import colored
-import colorama  # Required for Windows
+import ssl
 import shtab
+import shutil
 
-__version__ = "3.2.0"
-__client_specification__ = "1.5"
+__version__ = "3.4.1"
+__client_specification__ = "2.3"
 
 REQUEST_HEADERS = {'User-Agent': 'tldr-python-client'}
 PAGES_SOURCE_LOCATION = os.environ.get(
@@ -27,24 +28,35 @@ PAGES_SOURCE_LOCATION = os.environ.get(
 ).rstrip('/')
 DOWNLOAD_CACHE_LOCATION = os.environ.get(
     'TLDR_DOWNLOAD_CACHE_LOCATION',
-    'https://tldr-pages.github.io/assets/tldr.zip'
+    'https://github.com/tldr-pages/tldr/releases/latest/download/tldr.zip'
 )
 
 USE_NETWORK = int(os.environ.get('TLDR_NETWORK_ENABLED', '1')) > 0
 USE_CACHE = int(os.environ.get('TLDR_CACHE_ENABLED', '1')) > 0
 MAX_CACHE_AGE = int(os.environ.get('TLDR_CACHE_MAX_AGE', 24*7))
+CAFILE = None if os.environ.get('TLDR_CERT', None) is None else \
+    Path(os.environ.get('TLDR_CERT')).expanduser()
 
 URLOPEN_CONTEXT = None
 if int(os.environ.get('TLDR_ALLOW_INSECURE', '0')) == 1:
     URLOPEN_CONTEXT = ssl.create_default_context()
     URLOPEN_CONTEXT.check_hostname = False
     URLOPEN_CONTEXT.verify_mode = ssl.CERT_NONE
+elif CAFILE:
+    URLOPEN_CONTEXT = ssl.create_default_context(cafile=CAFILE)
 
 OS_DIRECTORIES = {
-    "linux": "linux",
+    "android": "android",
     "darwin": "osx",
+    "freebsd": "freebsd",
+    "linux": "linux",
+    "macos": "osx",
+    "netbsd": "netbsd",
+    "openbsd": "openbsd",
+    "osx": "osx",
     "sunos": "sunos",
-    "win32": "windows"
+    "win32": "windows",
+    "windows": "windows"
 }
 
 
@@ -75,28 +87,28 @@ def get_default_language() -> str:
     return default_lang
 
 
-def get_cache_dir() -> str:
+def get_cache_dir() -> Path:
     if os.environ.get('XDG_CACHE_HOME', False):
-        return os.path.join(os.environ.get('XDG_CACHE_HOME'), 'tldr')
+        return Path(os.environ.get('XDG_CACHE_HOME')) / 'tldr'
     if os.environ.get('HOME', False):
-        return os.path.join(os.environ.get('HOME'), '.cache', 'tldr')
-    return os.path.join(os.path.expanduser("~"), ".cache", "tldr")
+        return Path(os.environ.get('HOME')) / '.cache' / 'tldr'
+    return Path.home() / '.cache' / 'tldr'
 
 
-def get_cache_file_path(command: str, platform: str, language: str) -> str:
+def get_cache_file_path(command: str, platform: str, language: str) -> Path:
     pages_dir = "pages"
     if language and language != 'en':
         pages_dir += "." + language
-    return os.path.join(get_cache_dir(), pages_dir, platform, command) + ".md"
+    return get_cache_dir() / pages_dir / platform / f"{command}.md"
 
 
 def load_page_from_cache(command: str, platform: str, language: str) -> Optional[str]:
     try:
-        with open(get_cache_file_path(
+        with get_cache_file_path(
             command,
             platform,
-            language), 'rb'
-        ) as cache_file:
+            language
+        ).open('rb') as cache_file:
             cache_file_contents = cache_file.read()
         return cache_file_contents
     except Exception:
@@ -111,8 +123,8 @@ def store_page_to_cache(
 ) -> Optional[str]:
     try:
         cache_file_path = get_cache_file_path(command, platform, language)
-        os.makedirs(os.path.dirname(cache_file_path), exist_ok=True)
-        with open(cache_file_path, "wb") as cache_file:
+        cache_file_path.parent.mkdir(parents=True, exist_ok=True)
+        with cache_file_path.open("wb") as cache_file:
             cache_file.write(page)
     except Exception:
         pass
@@ -121,7 +133,7 @@ def store_page_to_cache(
 def have_recent_cache(command: str, platform: str, language: str) -> bool:
     try:
         cache_file_path = get_cache_file_path(command, platform, language)
-        last_modified = datetime.fromtimestamp(os.path.getmtime(cache_file_path))
+        last_modified = datetime.fromtimestamp(cache_file_path.stat().st_mtime)
         hours_passed = (datetime.now() - last_modified).total_seconds() / 3600
         return hours_passed <= MAX_CACHE_AGE
     except Exception:
@@ -197,7 +209,7 @@ def get_platform() -> str:
 
 
 def get_platform_list() -> List[str]:
-    platforms = ['common'] + list(OS_DIRECTORIES.values())
+    platforms = ['common'] + list(set(OS_DIRECTORIES.values()))
     current_platform = get_platform()
     platforms.remove(current_platform)
     platforms.insert(0, current_platform)
@@ -230,6 +242,74 @@ def get_language_list() -> List[str]:
     return languages
 
 
+def get_page_for_every_platform(
+    command: str,
+    remote: Optional[str] = None,
+    platforms: Optional[List[str]] = None,
+    languages: Optional[List[str]] = None
+) -> Union[List[Tuple[str, str]], bool]:
+    """Gives a list of tuples result-platform ordered by priority."""
+    if platforms is None:
+        platforms = get_platform_list()
+    else:
+        # When platform is explicitly specified, ensure 'common' is included as fallback
+        if 'common' not in platforms and len(platforms) > 0:
+            platforms = platforms + ['common']
+    if languages is None:
+        languages = get_language_list()
+    # only use cache
+    if USE_CACHE:
+        result = list()
+        for platform in platforms:
+            for language in languages:
+                if platform is None:
+                    continue
+                try:
+                    result.append(
+                        (get_page_for_platform(
+                                command,
+                                platform,
+                                remote,
+                                language,
+                                only_use_cache=True,
+                        ), platform)
+                    )
+                    break   # Don't want to look for the same page in other langs
+                except CacheNotExist:
+                    continue
+        if result:  # Return if smth was found
+            return result
+    # Know here that we don't have the info in cache
+    result = list()
+    for platform in platforms:
+        for language in languages:
+            if platform is None:
+                continue
+            try:
+                result.append(
+                    (
+                        get_page_for_platform(
+                            command,
+                            platform,
+                            remote,
+                            language
+                        ),
+                        platform
+                    )
+                )
+                break
+            except HTTPError as err:
+                if err.code != 404:
+                    raise
+            except URLError:
+                if not PAGES_SOURCE_LOCATION.startswith('file://'):
+                    raise
+    if result:  # Return if smth was found
+        return result
+
+    return False
+
+
 def get_page(
     command: str,
     remote: Optional[str] = None,
@@ -297,21 +377,33 @@ ACCEPTED_COLOR_ATTRS = [
 
 LEADING_SPACES_NUM = 2
 
+EXAMPLE_SPLIT_REGEX = re.compile(r'(?P<example>`.+?`)')
+EXAMPLE_REGEX = re.compile(r'(?:`)(?P<example>.+?)(?:`)')
 COMMAND_SPLIT_REGEX = re.compile(r'(?P<param>{{.+?}*}})')
 PARAM_REGEX = re.compile(r'(?:{{)(?P<param>.+?)(?:}})')
 
 
-def get_commands(platforms: Optional[List[str]] = None) -> List[str]:
+def get_commands(platforms: Optional[List[str]] = None,
+                 language: Optional[str] = None) -> List[str]:
     if platforms is None:
         platforms = get_platform_list()
 
+    if language:
+        languages = [get_language_code(language[0])]
+    else:
+        languages = get_language_list()
+
     commands = []
-    if os.path.exists(get_cache_dir()):
+    if get_cache_dir().exists():
         for platform in platforms:
-            path = os.path.join(get_cache_dir(), 'pages', platform)
-            if not os.path.exists(path):
-                continue
-            commands += [file[:-3] for file in os.listdir(path) if file.endswith(".md")]
+            for language in languages:
+                pages_dir = f'pages.{language}' if language != 'en' else 'pages'
+                path = get_cache_dir() / pages_dir / platform
+                if not path.exists():
+                    continue
+                commands += [f"{file.stem} ({language})"
+                             for file in path.iterdir()
+                             if file.suffix == '.md']
     return commands
 
 
@@ -331,7 +423,12 @@ def colors_of(key: str) -> Tuple[str, str, List[str]]:
     return (color, on_color, attrs)
 
 
-def output(page: str, plain: bool = False) -> None:
+def output(page: str, display_option_length: str, plain: bool = False) -> None:
+    def emphasise_example(x: str) -> str:
+        # Use ANSI escapes to enable italics at the start and disable at the end
+        # Also use the color yellow to differentiate from the default green
+        return "\x1B[3m" + colored(x.group('example'), 'yellow') + "\x1B[23m"
+
     if not plain:
         print()
     for line in page:
@@ -340,69 +437,131 @@ def output(page: str, plain: bool = False) -> None:
         if plain:
             print(line)
             continue
+
         elif len(line) == 0:
             continue
+
+        # Handle the command name
         elif line[0] == '#':
             line = ' ' * LEADING_SPACES_NUM + \
                 colored(line.replace('# ', ''), *colors_of('name')) + '\n'
             sys.stdout.buffer.write(line.encode('utf-8'))
+
+        # Handle the command description
         elif line[0] == '>':
             line = ' ' * (LEADING_SPACES_NUM - 1) + \
                 colored(
                     line.replace('>', '').replace('<', ''),
                     *colors_of('description')
-            )
+                )
             sys.stdout.buffer.write(line.encode('utf-8'))
+
+        # Handle an example description
         elif line[0] == '-':
-            line = '\n' + ' ' * LEADING_SPACES_NUM + \
-                colored(line, *colors_of('example'))
+
+            # Stylize text within backticks using yellow italics
+            if '`' in line:
+                elements = ['\n', ' ' * LEADING_SPACES_NUM]
+
+                for item in EXAMPLE_SPLIT_REGEX.split(line):
+                    item, replaced = EXAMPLE_REGEX.subn(emphasise_example, item)
+                    if not replaced:
+                        item = colored(item, *colors_of('example'))
+                    elements.append(item)
+
+                line = ''.join(elements)
+
+            # Otherwise, use the same colour for the whole line
+            else:
+                line = '\n' + ' ' * LEADING_SPACES_NUM + \
+                    colored(line, *colors_of('example'))
+
             sys.stdout.buffer.write(line.encode('utf-8'))
+
+        # Handle an example command
         elif line[0] == '`':
-            line = line[1:-1]  # need to actually parse ``
+            line = line[1:-1]  # Remove backticks for parsing
+
+            # Handle escaped placeholders first
+            line = line.replace(r'\{\{', '__ESCAPED_OPEN__')
+            line = line.replace(r'\}\}', '__ESCAPED_CLOSE__')
+
+            # Extract long or short options from placeholders
+            if display_option_length == "short":
+                line = re.sub(r'{{\[([^|]+)\|[^|]+?\]}}', r'\1', line)
+            elif display_option_length == "long":
+                line = re.sub(r'{{\[[^|]+\|([^|]+?)\]}}', r'\1', line)
+
             elements = [' ' * 2 * LEADING_SPACES_NUM]
             for item in COMMAND_SPLIT_REGEX.split(line):
                 item, replaced = PARAM_REGEX.subn(
-                    lambda x: colored(
-                        x.group('param'), *colors_of('parameter')),
+                    lambda x: colored(x.group('param'), *colors_of('parameter')),
                     item)
                 if not replaced:
                     item = colored(item, *colors_of('command'))
                 elements.append(item)
-            sys.stdout.buffer.write(''.join(elements).encode('utf-8'))
+
+            line = ''.join(elements)
+
+            # Restore escaped placeholders
+            line = line.replace('__ESCAPED_OPEN__', '{{')
+            line = line.replace('__ESCAPED_CLOSE__', '}}')
+
+            sys.stdout.buffer.write(line.encode('utf-8'))
         print()
     print()
 
 
 def update_cache(language: Optional[List[str]] = None) -> None:
-    if language is None:
-        tldr_language = os.environ.get("TLDR_LANGUAGE", get_default_language())
-        language = tldr_language if tldr_language else 'en'
-    elif isinstance(language, list):
-        language = language[0]
-    try:
-        pages_dir = "pages"
-        if language and language != 'en':
-            pages_dir += "." + language
-        req = urlopen(Request(
-            DOWNLOAD_CACHE_LOCATION,
-            headers=REQUEST_HEADERS
-        ), context=URLOPEN_CONTEXT)
-        zipfile = ZipFile(BytesIO(req.read()))
-        pattern = re.compile(r"{}/(.+)/(.+)\.md".format(pages_dir))
-        cached = 0
-        for entry in zipfile.namelist():
-            match = pattern.match(entry)
-            if match:
-                store_page_to_cache(
-                    zipfile.read(entry),
-                    match.group(2),
-                    match.group(1),
-                    language
-                )
-                cached += 1
-        print("Updated cache for {:d} entries".format(cached))
-    except Exception:
-        sys.exit("Error: Unable to update cache from " + DOWNLOAD_CACHE_LOCATION)
+    languages = get_language_list()
+    if language and language[0] not in languages:
+        languages.append(language[0])
+    for language in languages:
+        try:
+            cache_location = f"{DOWNLOAD_CACHE_LOCATION[:-4]}-pages.{language}.zip"
+            req = urlopen(Request(
+                cache_location,
+                headers=REQUEST_HEADERS
+            ), context=URLOPEN_CONTEXT)
+            zipfile = ZipFile(BytesIO(req.read()))
+            pattern = re.compile(r"(.+)/(.+)\.md")
+            cached = 0
+            for entry in zipfile.namelist():
+                match = pattern.match(entry)
+                if match:
+                    store_page_to_cache(
+                        zipfile.read(entry),
+                        match.group(2),
+                        match.group(1),
+                        language
+                    )
+                    cached += 1
+            print(
+                "Updated cache for language "
+                f"{language}: {cached} entries"
+            )
+        except Exception:
+            print(
+                "Error: Unable to update cache for language "
+                f"{language} from {cache_location}"
+            )
+
+
+def clear_cache(language: Optional[List[str]] = None) -> None:
+    languages = get_language_list()
+    if language and language[0] not in languages:
+        languages.append(language[0])
+    for language in languages:
+        pages_dir = f'pages.{language}' if language != 'en' else 'pages'
+        cache_dir = get_cache_dir() / pages_dir
+        if cache_dir.exists() and cache_dir.is_dir():
+            try:
+                shutil.rmtree(cache_dir)
+                print(f"Cleared cache for language {language}")
+            except Exception as e:
+                print(f"Error: Unable to delete cache directory {cache_dir}: {e}")
+        else:
+            print(f"No cache directory found for language {language}")
 
 
 def create_parser() -> ArgumentParser:
@@ -425,18 +584,25 @@ def create_parser() -> ArgumentParser:
                         type=str,
                         help="Search for a specific command from a query")
 
-    parser.add_argument('-u', '--update_cache',
+    parser.add_argument('-u', '--update', '--update_cache',
                         action='store_true',
                         help="Update the local cache of pages and exit")
 
+    parser.add_argument('-k', '--clear-cache',
+                        action='store_true',
+                        help="Delete the local cache of pages and exit")
+
     parser.add_argument(
         '-p', '--platform',
         nargs=1,
         default=None,
         type=str,
-        choices=['linux', 'osx', 'sunos', 'windows', 'common'],
+        choices=['android', 'freebsd', 'linux', 'netbsd', 'openbsd', 'osx', 'sunos',
+                 'windows', 'common'],
         metavar='PLATFORM',
-        help="Override the operating system [linux, osx, sunos, windows, common]"
+        help="Override the operating system "
+             "[android, freebsd, linux, netbsd, openbsd,"
+             " osx, sunos, windows, common]"
     )
 
     parser.add_argument('-l', '--list',
@@ -472,6 +638,16 @@ def create_parser() -> ArgumentParser:
                         action='store_true',
                         help='Just print the plain page file.')
 
+    parser.add_argument('--short-options',
+                        default=False,
+                        action="store_true",
+                        help='Display shortform options over longform')
+
+    parser.add_argument('--long-options',
+                        default=False,
+                        action="store_true",
+                        help='Display longform options over shortform')
+
     parser.add_argument(
         'command', type=str, nargs='*', help="command to lookup", metavar='command'
     ).complete = {"bash": "shtab_tldr_cmd_list", "zsh": "shtab_tldr_cmd_list"}
@@ -493,80 +669,95 @@ def main() -> None:
 
     options = parser.parse_args()
 
-    colorama.init(strip=options.color)
+    display_option_length = "long"
+    if not (options.short_options or options.long_options):
+        if os.environ.get('TLDR_OPTIONS') == "short":
+            display_option_length = "short"
+        elif os.environ.get('TLDR_OPTIONS') == "long":
+            display_option_length = "long"
+        elif os.environ.get('TLDR_OPTIONS') == "both":
+            display_option_length = "both"
+    if options.short_options:
+        display_option_length = "short"
+    if options.long_options:
+        display_option_length = "long"
+    if options.short_options and options.long_options:
+        display_option_length = "both"
+    if sys.platform == "win32":
+        import colorama
+        colorama.init(strip=options.color)
+
     if options.color is False:
         os.environ["FORCE_COLOR"] = "true"
 
-    if options.update_cache:
+    if options.update:
         update_cache(language=options.language)
         return
     elif len(sys.argv) == 1:
         parser.print_help(sys.stderr)
         sys.exit(1)
+    if options.clear_cache:
+        clear_cache(language=options.language)
+        return
+    elif len(sys.argv) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
     if options.list:
-        print(get_commands(options.platform))
+        print('\n'.join(get_commands(options.platform, options.language)))
     elif options.render:
         for command in options.command:
-            if os.path.exists(command):
-                with open(command, encoding='utf-8') as open_file:
+            file_path = Path(command)
+            if file_path.exists():
+                with file_path.open(encoding='utf-8') as open_file:
                     output(open_file.read().encode('utf-8').splitlines(),
+                           display_option_length,
                            plain=options.markdown)
     elif options.search:
-        command = '-'.join(options.command)
-        page = None
-        maxprob = 0
-        searchquery = options.search.split(' ')
-
-        platforms = get_platform_list()
-        for i in get_commands(platforms):
-            if i.startswith(command):
-                for p in platforms:
-                    result = load_page_from_cache(i, p, options.language)
-                    if result is not None and have_recent_cache(i, p, options.language):
-                        break
-                if result is None:
-                    raise SystemExit("Please update cache")
-                result = result.decode("utf-8")
-                result = result.split()
-                for word in searchquery:
-                    prob = 0
-                    for line in result:
-                        line = line.lower()
-                        if word in line:
-                            prob += 1
-                            if line.startswith("#"):
-                                prob += 7
-                            elif line.startswith(">"):
-                                prob += 5
-                            elif line.startswith("-"):
-                                prob += 2
-                    if prob > maxprob:
-                        maxprob = prob
-                        page = i
-
-        if page:
-            result = get_page(page, None, options.platform, options.language)
-            output(result, plain=options.markdown)
+        search_term = options.search.lower()
+        commands = get_commands(options.platform, options.language)
+        if not commands:
+            print("Update cache, no commands to check from.")
+            return
+        similar_commands = []
+        for command in commands:
+            if search_term in command.lower():
+                similar_commands.append(command)
+        if similar_commands:
+            print("Similar commands found:")
+            print('\n'.join(similar_commands))
+            return
         else:
-            print("No results found")
-
+            print("No commands matched your search term.")
     else:
         try:
             command = '-'.join(options.command).lower()
-            result = get_page(
+            results = get_page_for_every_platform(
                 command,
                 options.source,
                 options.platform,
                 options.language
             )
-            if not result:
+            if not results:
                 sys.exit((
                     "`{cmd}` documentation is not available.\n"
                     "If you want to contribute it, feel free to"
                     " send a pull request to: https://github.com/tldr-pages/tldr"
                 ).format(cmd=command))
             else:
-                output(result, plain=options.markdown)
+                output(results[0][0], display_option_length, plain=options.markdown)
+                if results[1:]:
+                    platforms_str = [result[1] for result in results[1:]]
+                    are_multiple_platforms = len(platforms_str) > 1
+                    if are_multiple_platforms:
+                        print(
+                            f"Found {len(platforms_str)} pages with the same name"
+                            f" under the platforms: {', '.join(platforms_str)}."
+                        )
+                    else:
+                        print(
+                            f"Found 1 page with the same name"
+                            f" under the platform: {platforms_str[0]}."
+                        )
         except URLError as e:
             sys.exit("Error fetching from tldr: {}".format(e))
 
